@@ -7,8 +7,9 @@ import sys
 import threading
 import time
 
+import pandas as pd
+
 try:
-    from gnuradio import analog  # pytype: disable=import-error
     from gnuradio import blocks  # pytype: disable=import-error
     from gnuradio import fft  # pytype: disable=import-error
     from gnuradio import gr  # pytype: disable=import-error
@@ -37,6 +38,7 @@ class grscan(gr.top_block):
         sdr="ettus",
         sdrargs=None,
         fft_size=1024,
+        retune_intervals=1,
         habets39=None,
     ):
         gr.top_block.__init__(self, "scan", catch_exceptions=True)
@@ -50,25 +52,89 @@ class grscan(gr.top_block):
         self.freq_update = 0
         self.no_freq_updates = 0
         self.retune_hz = retune_hz
+        self.retune_intervals = retune_intervals
+        self.retune_timed_hz = retune_hz / self.retune_intervals
 
         ##################################################
         # Variables
         ##################################################
-        self.sweep_freq = sweep_freq = 1 / sweep_sec
-        self.scan_samp_rate = scan_samp_rate = 256e3
         self.fft_size = fft_size
         self.center_freq = freq_start
 
         ##################################################
         # Blocks
         ##################################################
-        self.blocks_probe_signal_x_0 = blocks.probe_signal_f()
 
-        def _center_freq_probe():
+        def _sweep_pos_to_freq(tune_time):  # nosemgrep
+            sweep_pos = (tune_time % self.sweep_sec) / self.sweep_sec
+            freq = ((self.freq_end - self.freq_start) * sweep_pos) + self.freq_start
+            return freq
+
+        def _retune_worker():
+            freq_steps = []
+            sweeps = 0
+            retune_interval = 1.0 / self.retune_hz
             while True:
-                val = self.blocks_probe_signal_x_0.level()
-                self.set_center_freq(val)
-                time.sleep(1.0 / self.retune_hz)
+                rollover = False
+                host_now = time.time()
+                sdr_now = self.get_sdr_time_now(self.source_0)
+                for i in range(self.retune_intervals):
+                    tune_time = sdr_now + (i * retune_interval)
+                    freq = _sweep_pos_to_freq(tune_time)
+                    # logging.info(
+                    #    "tune_time: %f now diff: %f %f %f MHz",
+                    #    tune_time,
+                    #    tune_time - sdr_now,
+                    #    i,
+                    #    freq / 1e6,
+                    # )
+                    if freq_steps and freq < freq_steps[-1]:
+                        rollover = True
+                        sweeps += 1
+                    freq_steps.append(freq)
+                    if tune_time == sdr_now:
+                        self.set_center_freq(freq)
+                    else:
+                        self.set_command_time(self.source_0, tune_time)
+                        self.set_center_freq(freq)
+                        self.clear_command_time(self.source_0)
+                if rollover and sweeps > 1:
+                    freq_df = pd.DataFrame(freq_steps, columns=["freq"]).sort_values(
+                        by=["freq"]
+                    )
+                    freq_df["freq"] /= 1e6
+                    freq_df["diff"] = freq_df["freq"].diff()
+                    dmin, dmean, dmax = (
+                        freq_df["diff"].min(),
+                        freq_df["diff"].mean(),
+                        freq_df["diff"].max(),
+                    )
+                    freq_max = freq_df[freq_df["diff"] == dmax]["freq"].iat[0]
+                    logging.info(
+                        "tuning step min %f MHz mean %f MHz max %f MHz (at %f MHz)",
+                        dmin,
+                        dmean,
+                        dmax,
+                        freq_max,
+                    )
+                    if dmean > samp_rate / 1e6 / 2:
+                        logging.warning(
+                            "mean tuning step is greater than --samp-rate/2"
+                        )
+                remainder_interval_time = (1.0 / self.retune_timed_hz) - (
+                    time.time() - host_now
+                )
+                if remainder_interval_time > 0:
+                    time.sleep(remainder_interval_time)
+                else:
+                    logging.info(
+                        "retune interval ran late near %.3f MHz by %.3fs (wanted interval %.3fs)",
+                        freq_steps[-1] / 1e6,
+                        abs(remainder_interval_time),
+                        retune_interval,
+                    )
+                if rollover:
+                    freq_steps = []
 
         logging.info(f"will scan from {freq_start} to {freq_end}")
         get_source(
@@ -81,9 +147,9 @@ class grscan(gr.top_block):
             sdrargs=sdrargs,
         )
 
-        self.center_freq_thread = threading.Thread(target=_center_freq_probe)
-        self.center_freq_thread.daemon = True
-        self.center_freq_thread.start()
+        self.retune_worker_thread = threading.Thread(target=_retune_worker)
+        self.retune_worker_thread.daemon = True
+        self.retune_worker_thread.start()
 
         self.habets39_sweepsinkv_0 = None
         if habets39:
@@ -93,35 +159,18 @@ class grscan(gr.top_block):
         self.fft_vxx_0 = fft.fft_vcc(
             fft_size, True, window.blackmanharris(fft_size), True, 1
         )
-        self.blocks_throttle_0 = blocks.throttle(gr.sizeof_float, scan_samp_rate, True)
         self.blocks_stream_to_vector_0 = blocks.stream_to_vector(
             gr.sizeof_gr_complex, fft_size
         )
-        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_ff(
-            freq_end - freq_start
-        )
         zmq_addr = f"tcp://{logaddr}:{logport}"
         logging.info("serving FFT on %s", zmq_addr)
         self.zeromq_pub_sink_0 = zeromq.pub_sink(1, 1, zmq_addr, 100, False, -1, "")
         self.blocks_complex_to_mag_0 = blocks.complex_to_mag(fft_size)
-        self.blocks_add_const_vxx_0 = blocks.add_const_ff(freq_start)
-        self.analog_sig_source_x_0 = analog.sig_source_f(
-            scan_samp_rate, analog.GR_SAW_WAVE, sweep_freq, 1, 0, 0
-        )
+        self.blocks_complex_to_mag_0.set_max_output_buffer(16)
 
         ##################################################
         # Connections
         ##################################################
-        # Tuning chain
-        self.connect(
-            (self.analog_sig_source_x_0, 0), (self.blocks_multiply_const_vxx_0, 0)
-        )
-        self.connect(
-            (self.blocks_multiply_const_vxx_0, 0), (self.blocks_add_const_vxx_0, 0)
-        )
-        self.connect((self.blocks_add_const_vxx_0, 0), (self.blocks_throttle_0, 0))
-        self.connect((self.blocks_throttle_0, 0), (self.blocks_probe_signal_x_0, 0))
-
         # FFT chain
         if self.habets39_sweepsinkv_0:
             self.connect((self.blocks_stream_to_vector_0, 0), (self.fft_vxx_0, 0))
