@@ -11,6 +11,7 @@ import time
 import bjoern
 import falcon
 import jinja2
+import numpy as np
 import pandas as pd
 import requests
 import schedule
@@ -33,6 +34,13 @@ from gamutrf.utils import rotate_file_n, SCAN_FRES
 MB = int(1.024e6)
 FFT_BUFFER_TIME = 3
 BUFF_FILE = "/dev/shm/scanfftbuffer.txt.zst"  # nosec
+PEAK_TRIGGER = int(os.environ.get("PEAK_TRIGGER", "0"))
+PIN_TRIGGER = int(os.environ.get("PIN_TRIGGER", "17"))
+if PEAK_TRIGGER == 1:
+    import RPi.GPIO as GPIO
+
+    GPIO.setmode(GPIO.BOARD)
+    GPIO.setup(PIN_TRIGGER, GPIO.OUT)
 
 PEAK_DBS = {}
 
@@ -53,7 +61,7 @@ def error_response(resp, text="error!"):
 
 def load_template(name):
     path = os.path.join("templates", name)
-    with open(os.path.abspath(path), "r") as fp:
+    with open(os.path.abspath(path), "r", encoding="utf-8") as fp:
         return jinja2.Template(fp.read())
 
 
@@ -160,7 +168,7 @@ def update_prom_vars(peak_dbs, new_bins, old_bins, prom_vars):
         old_bins_prom.labels(bin_freq=obin).inc()
 
 
-def process_fft(args, prom_vars, ts, fftbuffer, lastbins):
+def process_fft(args, prom_vars, ts, fftbuffer, lastbins, running_df):
     global PEAK_DBS
     df = pd.DataFrame(fftbuffer, columns=["ts", "freq", "db"])
     # resample to SCAN_FRES
@@ -199,9 +207,32 @@ def process_fft(args, prom_vars, ts, fftbuffer, lastbins):
         df, width=args.width, prominence=args.prominence, threshold=args.threshold
     )
 
+    if PEAK_TRIGGER == 1 and signals:
+        led_sleep = 0.2
+        GPIO.output(PIN_TRIGGER, GPIO.HIGH)
+        time.sleep(led_sleep)
+        GPIO.output(PIN_TRIGGER, GPIO.LOW)
+
+    if running_df is None:
+        running_df = df
+    else:
+        now = time.time()
+        running_df = running_df[running_df.ts >= (now - args.running_fft_secs)]
+        running_df = pd.concat(running_df, df)
+    mean_running_df = running_df[["freq", "db"]].groupby(["freq"]).mean().reset_index()
+    sample_count_df = df[["freq"]].copy()
+    sample_count_df["freq"] = np.floor(sample_count_df["freq"])
+    # nosemgrep
+    sample_count_df["size"] = sample_count_df.groupby("freq").transform("size")
+    sample_count_df["size"] = abs(
+        sample_count_df["size"].mean() - sample_count_df["size"]
+    )
+    sample_count_df["size"].iat[0] = 0
+    sample_count_df["size"].iat[-1] = 0
+
     if args.fftgraph:
         rotate_file_n(args.fftgraph, args.nfftgraph)
-        graph_fft_peaks(args.fftgraph, df, signals)
+        graph_fft_peaks(args.fftgraph, df, mean_running_df, sample_count_df, signals)
 
     for peak_freq, peak_db in signals:
         center_freq = get_center(
@@ -298,6 +329,7 @@ def process_fft_lines(
     fft_packets = 0
     max_scan_pos = 0
     context = zstandard.ZstdDecompressor()
+    running_df = None
 
     while True:
         if os.path.exists(args.log):
@@ -307,7 +339,7 @@ def process_fft_lines(
             logging.info(f"opening {args.log}")
             mode = "w"
         openlogts = int(time.time())
-        with open(args.log, mode=mode) as l:
+        with open(args.log, mode=mode, encoding="utf-8") as l:
             while True:
                 if not proxy_result.running():
                     logging.error(
@@ -372,7 +404,12 @@ def process_fft_lines(
                         max_scan_pos = row.scan_pos
                         frame_counter.inc()
                         new_lastbins = process_fft(
-                            args, prom_vars, row.ts, fftbuffer, lastbins
+                            args,
+                            prom_vars,
+                            row.ts,
+                            fftbuffer,
+                            lastbins,
+                            running_df,
                         )
                         if new_lastbins is not None:
                             lastbins = new_lastbins
@@ -556,6 +593,14 @@ def argument_parser():
         default=1,
         help="Max number of recordings per worker to request",
     )
+    parser.add_argument(
+        "--running_fft_secs",
+        dest="running_fft_secs",
+        type=int,
+        default=900,
+        help="Number of seconds for running FFT average",
+    )
+
     return parser
 
 
